/*
 * Copyright 2011, Maarten Baert <maarten-baert@hotmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef MEMORYBUFFER_H
#define MEMORYBUFFER_H

#include "SharedStreamBuffers.h"

extern SharedBufferInterface sbi_memorybuffer;

class MemoryBuffer : public SharedBuffer {
	
	private:
	char *data;
	unsigned int length, capacity;
	unsigned int readpos, writepos;
	
	private:
	void Clear();
	
	public:
	// implementation-specific functions
	static MemoryBuffer* Create();
	static void Destroy(MemoryBuffer* buffer);
	bool ReadFromFile(const char* filename);
	bool WriteToFile(const char* filename);
	
	// functions inherited from SharedStream
	// these functions take a SharedStream* as the first argument because the interface
	// requires this, however the actual object should be a MemoryBuffer
	// (static because SharedStreamInterface uses function pointers)
	static void ReadData(SharedStream* stream, void* ptr, uint32_t bytes);
	static void WriteData(SharedStream* stream, const void* ptr, uint32_t bytes);
	static uint32_t GetBytesLeft(const SharedStream* stream);
	
	// functions inherited from SharedBuffer
	// these functions take a SharedBuffer* as the first argument because the interface
	// requires this, however the actual object should be a MemoryBuffer
	// (static because SharedBufferInterface uses function pointers)
	static uint32_t GetReadPos(const SharedBuffer* buffer);
	static uint32_t GetWritePos(const SharedBuffer* buffer);
	static uint32_t GetLength(const SharedBuffer* buffer);
	static void SetReadPos(SharedBuffer* buffer, uint32_t pos);
	static void SetWritePos(SharedBuffer* buffer, uint32_t pos);
	static void SetLength(SharedBuffer* buffer, uint32_t length);
	
};

#endif // MEMORYBUFFER_H
