/*
 * Copyright 2011, Maarten Baert <maarten-baert@hotmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "MemoryBuffer.h"

#include <map>

SharedStreamBufferInterface ssbm_interface;
std::map<uint32_t, SharedStream*> ssbm_streams;
std::map<uint32_t, SharedBuffer*> ssbm_buffers;
uint32_t ssbm_streambuffer_idcounter = 0;

// This class keeps a list of all buffers and streams. It doesn't have any
// member variables because all functions have to be static.
class SharedStreamBufferManager {
	
	public:
	SharedStreamBufferManager() {
		ssbm_interface.registerstream = &RegisterStream;
		ssbm_interface.unregisterstream = &UnregisterStream;
		ssbm_interface.findstream = &FindStream;
		ssbm_interface.registerbuffer = &RegisterBuffer;
		ssbm_interface.unregisterbuffer = &UnregisterBuffer;
		ssbm_interface.findbuffer = &FindBuffer;
	}
	static void RegisterStream(SharedStream* stream) {
		ssbm_streams.insert(std::pair<uint32_t, SharedStream*>(++ssbm_streambuffer_idcounter, stream));
		stream->id = ssbm_streambuffer_idcounter;
	}
	static void UnregisterStream(SharedStream* stream) {
		ssbm_streams.erase(stream->id);
	}
	static SharedStream* FindStream(uint32_t id) {
		// First search for a stream with this id.
		// If it doesn't exist, call FindBuffer(id).
		std::map<uint32_t, SharedStream*>::iterator it = ssbm_streams.find(id);
		return (it == ssbm_streams.end())? FindBuffer(id) : it->second;
	}
	static void RegisterBuffer(SharedBuffer* buffer) {
		ssbm_buffers.insert(std::pair<uint32_t, SharedBuffer*>(++ssbm_streambuffer_idcounter, buffer));
		buffer->id = ssbm_streambuffer_idcounter;
	}
	static void UnregisterBuffer(SharedBuffer* buffer) {
		ssbm_buffers.erase(buffer->id);
	}
	static SharedBuffer* FindBuffer(uint32_t id) {
		std::map<uint32_t, SharedBuffer*>::iterator it = ssbm_buffers.find(id);
		return (it == ssbm_buffers.end())? NULL : it->second;
	}
	
} sharedstreambuffermanager;

