/*
 * Copyright 2011, Maarten Baert <maarten-baert@hotmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "gm.h"
#include "SharedStreamBuffers.h"

extern SharedStreamBufferInterface ssbm_interface;

gmexport double buffer_exists(double id) {
	SharedBuffer *buffer = ssbm_interface.FindBuffer(gm_double_to_uint(id));
	return (buffer != NULL)? 1 : 0;
}

gmexport double buffer_get_read_pos(double id) {
	SharedBuffer *buffer = ssbm_interface.FindBuffer(gm_double_to_uint(id));
	if(buffer == NULL) return 0;
	return buffer->GetReadPos();
}

gmexport double buffer_get_write_pos(double id) {
	SharedBuffer *buffer = ssbm_interface.FindBuffer(gm_double_to_uint(id));
	if(buffer == NULL) return 0;
	return buffer->GetWritePos();
}

gmexport double buffer_get_length(double id) {
	SharedBuffer *buffer = ssbm_interface.FindBuffer(gm_double_to_uint(id));
	if(buffer == NULL) return 0;
	return buffer->GetLength();
}

gmexport double buffer_set_read_pos(double id, double readpos) {
	SharedBuffer *buffer = ssbm_interface.FindBuffer(gm_double_to_uint(id));
	if(buffer == NULL) return 0;
	buffer->SetReadPos(gm_double_to_uint(readpos));
	return 1;
}

gmexport double buffer_set_write_pos(double id, double writepos) {
	SharedBuffer *buffer = ssbm_interface.FindBuffer(gm_double_to_uint(id));
	if(buffer == NULL) return 0;
	buffer->SetWritePos(gm_double_to_uint(writepos));
	return 1;
}

gmexport double buffer_set_length(double id, double length) {
	SharedBuffer *buffer = ssbm_interface.FindBuffer(gm_double_to_uint(id));
	if(buffer == NULL) return 0;
	buffer->SetLength(gm_double_to_uint(length));
	return 1;
}

