/*
 * Copyright 2011, Maarten Baert <maarten-baert@hotmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "gm.h"
#include "SharedStreamBuffers.h"

extern SharedStreamBufferInterface ssbm_interface;

gmexport double stream_exists(double id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	return (stream != NULL)? 1 : 0;
}

template<typename X>
inline X stream_read_x(SharedStream* stream) {
	X value;
	stream->ReadData(&value, sizeof(X));
	return value;
}
template<typename X>
inline void stream_write_x(SharedStream* stream, X value) {
	stream->WriteData(&value, sizeof(X));
}

gmexport double stream_read_int8(double id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	return (double)(stream_read_x<int8_t>(stream));
}
gmexport double stream_read_uint8(double id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	return (double)(stream_read_x<uint8_t>(stream));
}
gmexport double stream_read_int16(double id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	return (double)(stream_read_x<int16_t>(stream));
}
gmexport double stream_read_uint16(double id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	return (double)(stream_read_x<uint16_t>(stream));
}
gmexport double stream_read_int32(double id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	return (double)(stream_read_x<int32_t>(stream));
}
gmexport double stream_read_uint32(double id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	return (double)(stream_read_x<uint32_t>(stream));
}
gmexport double stream_read_int64(double id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	return (double)(stream_read_x<int64_t>(stream));
}
gmexport double stream_read_uint64(double id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	return (double)(stream_read_x<uint64_t>(stream));
}
gmexport double stream_read_float32(double id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	return (double)(stream_read_x<float>(stream));
}
gmexport double stream_read_float64(double id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	return stream_read_x<double>(stream);
}

gmexport double stream_write_int8(double id, double value) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	stream_write_x<int8_t>(stream, (int8_t)(gm_double_to_int(value)));
	return 1;
}
gmexport double stream_write_uint8(double id, double value) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	stream_write_x<uint8_t>(stream, (uint8_t)(gm_double_to_uint(value)));
	return 1;
}
gmexport double stream_write_int16(double id, double value) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	stream_write_x<int16_t>(stream, (int16_t)(gm_double_to_int(value)));
	return 1;
}
gmexport double stream_write_uint16(double id, double value) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	stream_write_x<uint16_t>(stream, (uint16_t)(gm_double_to_uint(value)));
	return 1;
}
gmexport double stream_write_int32(double id, double value) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	stream_write_x<int32_t>(stream, (int32_t)(gm_double_to_int(value)));
	return 1;
}
gmexport double stream_write_uint32(double id, double value) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	stream_write_x<uint32_t>(stream, (uint32_t)(gm_double_to_uint(value)));
	return 1;
}
gmexport double stream_write_int64(double id, double value) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	stream_write_x<int64_t>(stream, (int64_t)(gm_double_to_longlong(value)));
	return 1;
}
gmexport double stream_write_uint64(double id, double value) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	stream_write_x<uint64_t>(stream, (uint64_t)(gm_double_to_ulonglong(value)));
	return 1;
}
gmexport double stream_write_float32(double id, double value) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	stream_write_x<float>(stream, gm_double_to_float(value));
	return 1;
}
gmexport double stream_write_float64(double id, double value) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	stream_write_x<double>(stream, value);
	return 1;
}

gmexport const char* stream_read_string(double id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return "";
	gm_returnstring.clear();
	char c;
	do {
		stream->ReadData(&c, 1);
		gm_returnstring.push_back(c);
	} while(c != '\0');
	return &(gm_returnstring.front());
}
gmexport double stream_write_string(double id, const char* string) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	stream->WriteData(string, strlen(string) + 1);
	return 1;
}

gmexport const char* stream_read_data(double id, double bytes) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return "";
	unsigned int l = gm_double_to_uint(bytes);
	char *str = gm_make_return_string(l);
	stream->ReadData(str, l);
	return str;
}
gmexport double stream_write_data(double id, const char* string) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	stream->WriteData(string, strlen(string));
	return 1;
}

gmexport const char* stream_read_hex(double id, double bytes) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return "";
	unsigned int l = gm_double_to_uint(bytes);
	char *str = gm_make_return_string(l * 2);
	for(unsigned int i = 0; i < l; ++i) {
		unsigned char a;
		stream->ReadData(&a, 1);
		str[2 * i]     = ((a >> 4) > 9)? 'a' + (a >> 4) - 10 : '0' + (a >> 4);
		str[2 * i + 1] = ((a & 15) > 9)? 'a' + (a & 15) - 10 : '0' + (a & 15);
	}
	return str;
}
gmexport double stream_write_hex(double id, const char* string) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	unsigned int l = strlen(string) / 2;
	for(unsigned int i = 0; i < l; ++i) {
		unsigned char a = string[2 * i];
		unsigned char b = string[2 * i + 1];
		unsigned char c =
			((('0' <= a && a <= '9')? a - '0' : (('a' <= a && a <= 'f')? 10 + a - 'a' : (('A' <= a && a <= 'F')? 10 + a - 'A' : 0))) << 4) |
			(('0' <= b && b <= '9')? b - '0' : (('a' <= b && b <= 'f')? 10 + b - 'a' : (('A' <= b && b <= 'F')? 10 + b - 'A' : 0)));
		stream->WriteData(&c, 1);
	}
	return 1;
}

gmexport double stream_write_stream(double id, double source_id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	SharedStream *source = ssbm_interface.FindStream(gm_double_to_uint(source_id));
	if(source == NULL) return 0;
	unsigned int l = source->GetBytesLeft();
	std::vector<char> data(l);
	source->ReadData(&(data.front()), l);
	stream->WriteData(&(data.front()), l);
	return 1;
}
gmexport double stream_write_stream_part(double id, double source_id, double bytes) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	SharedStream *source = ssbm_interface.FindStream(gm_double_to_uint(source_id));
	if(source == NULL) return 0;
	unsigned int l = gm_double_to_uint(bytes);
	unsigned int bytesleft = source->GetBytesLeft();
	if(l > bytesleft) {
		l = bytesleft;
	}
	std::vector<char> data(l);
	source->ReadData(&(data.front()), l);
	stream->WriteData(&(data.front()), l);
	return 1;
}

gmexport double stream_get_bytes_left(double id) {
	SharedStream *stream = ssbm_interface.FindStream(gm_double_to_uint(id));
	if(stream == NULL) return 0;
	return stream->GetBytesLeft();
}

